/*
Copyright (c) 2007, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.3.0
*/
/* Panel modality mask styles */

.yui-skin-sam .mask {

    background-color: #000;
    opacity: .25;
    *filter: alpha(opacity=25);  /* Set opacity in IE */

}


/* Panel styles */

.yui-skin-sam .yui-panel-container {

    padding: 0 1px;
    *padding: 2px 3px;

}

.yui-skin-sam .yui-panel {

	position: relative;
	*zoom: 1;
	left: 0;
	top: 0;
    border-style: solid;
    border-width: 1px 0;
    border-color: #808080;
	z-index: 1;

}

.yui-skin-sam .yui-panel .hd, 
.yui-skin-sam .yui-panel .bd, 
.yui-skin-sam .yui-panel .ft {

    /*
        Use of "zoom: 1" is to trigger "haslayout" for IE to get 
        negative margins working.
    */
    
    *zoom: 1;

    /*
        Use of "position: relative" is necessary to get negative margins 
        working in IE.
    */

    *position: relative; 

    border-style: solid;
    border-width: 0 1px;
    border-color: #808080;
    margin: 0 -1px;
  
}

.yui-skin-sam .yui-panel .hd {

    border-bottom: solid 1px #ccc;

}

.yui-skin-sam .yui-panel .bd,
.yui-skin-sam .yui-panel .ft {

    background-color: #F2F2F2;

}

.yui-skin-sam .yui-panel .hd {

    padding: 0 10px;
    font-size: 93%;  /* 12px */
    line-height: 2;  /* ~24px */
    *line-height: 1.9; /* For IE */
    font-weight: bold;
    color: #000;
    background: url(../../../../assets/skins/sam/sprite.png) repeat-x 0 -200px;
    
}

.yui-skin-sam .yui-panel .bd {

    padding: 10px;

}

.yui-skin-sam .yui-panel .ft {

    border-top: solid 1px #808080;
    padding: 5px 10px;
    font-size: 77%;

}

.yui-skin-sam .yui-panel-container.focused .yui-panel .hd {

}

.yui-skin-sam .container-close {

    position: absolute;
    top: 5px;
    right: 6px;
    width: 25px;
    height: 15px;
    background: url(../../../../assets/skins/sam/sprite.png) no-repeat 0 -300px;

}


/* Panel underlay styles */

.yui-skin-sam .yui-panel-container .underlay {

    right: -1px;
    left: -1px;

}

.yui-skin-sam .yui-panel-container.matte {

	padding: 9px 10px;
	background-color: #fff;

}

.yui-skin-sam .yui-panel-container.shadow {

    /* IE 7 Quirks Mode and IE 6 Standards Mode and Quirks mode */
    _padding: 2px 5px 0 3px; 

}

.yui-skin-sam .yui-panel-container.shadow .underlay {

	position: absolute;

    top: 2px;
    right: -3px;
    bottom: -3px;
    left: -3px;

    /* IE 7 Standards Mode */

    *top: 3px;
    *left: -1px;
    *right: -1px;
    *bottom: -1px;

    /* IE 7 Quirks Mode and IE 6 Standards Mode and Quirks mode */

    _top: 0;
    _right: 0;
    _bottom: 0;
    _left: 0;
    _margin-top: 3px;
    _margin-left: -1px;

    background-color: #000;
    opacity: .12;
    *filter: alpha(opacity=12);  /* Set opacity in IE */

}


/* Dialog styles */

.yui-skin-sam .yui-dialog .ft {

    border-top: none;
    padding: 0 10px 10px 10px;
    font-size: 100%;

}

.yui-skin-sam .yui-dialog .ft .button-group {

    display: block;
    text-align: right;

}


/* Dialog default button styles */

.yui-skin-sam .yui-dialog .ft .default {

    border-color: #304369;
    background-position: 0 -1400px;

}

.yui-skin-sam .yui-dialog .ft .default .first-child {

    border-color: #304369;

}

.yui-skin-sam .yui-dialog .ft .default button {

    color: #fff;

}


/* SimpleDialog icon styles */

.yui-skin-sam .yui-simple-dialog .bd .yui-icon {

    background: url(../../../../assets/skins/sam/sprite.png) no-repeat 0 0;
	width: 16px;
	height: 16px;
	margin-right: 10px;
	float: left;

}

.yui-skin-sam .yui-simple-dialog .bd span.blckicon {

    background-position: 0 -1100px;

}

.yui-skin-sam .yui-simple-dialog .bd span.alrticon {

    background-position: 0 -1050px;

}

.yui-skin-sam .yui-simple-dialog .bd span.hlpicon {

    background-position: 0 -1150px;

}

.yui-skin-sam .yui-simple-dialog .bd span.infoicon {

    background-position: 0 -1200px;

}

.yui-skin-sam .yui-simple-dialog .bd span.warnicon {

    background-position: 0 -1900px;

}

.yui-skin-sam .yui-simple-dialog .bd span.tipicon {

    background-position: 0 -1250px;

}


/* Tooltip styles */

.yui-skin-sam .yui-tt .bd {

    position: relative;
    top: 0;
    left: 0;
    z-index: 1;
    color: #000;
    padding: 2px 5px;
    border-color: #D4C237 #A6982B #A6982B #A6982B;
    border-width: 1px;
    border-style: solid;
    background-color: #FFEE69;

}

.yui-skin-sam .yui-tt.show-scrollbars .bd {

    overflow: auto;

}

.yui-skin-sam .yui-tt-shadow {

    top: 2px;
    right: -3px;
    left: -3px;
    bottom: -3px;
    background-color: #000;

}

.yui-skin-sam .yui-tt-shadow-visible {

    opacity: .12;
    *filter: alpha(opacity=12);  /* For IE */    

}