/*
Copyright (c) 2007, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.3.0
*/
/**
 * CORE
 *
 * This is the set of CSS rules required by Calendar to drive core functionality and structure.
 * Changes to these rules may result in the Calendar not functioning or rendering correctly.
 *
 * They should not be modified for skinning.
 **/
 
/* CALENDAR BOUNDING BOX */
.yui-calcontainer {
	position:relative;
	float:left;
	_overflow:hidden; /* IE6 only, to clip iframe shim */
}

/* IFRAME SHIM */
.yui-calcontainer iframe {
	position:absolute;
	border:none;
	margin:0;padding:0;
	z-index:0;
	width:100%;
	height:100%;
	left:0px;
	top:0px;
}

/* IFRAME SHIM IE6 only */
.yui-calcontainer iframe.fixedsize {
	width:50em;
	height:50em;
	top:-1px;
	left:-1px;
}

/* BOUNDING BOX FOR EACH CALENDAR GROUP PAGE */
.yui-calcontainer.multi .groupcal {
	z-index:1;
	float:left;
	position:relative;
}

/* TITLE BAR */
.yui-calcontainer .title {
	position:relative;
	z-index:1;
}

/* CLOSE ICON CONTAINER */
.yui-calcontainer .close-icon {
	position:absolute;
	z-index:1;
}

/* CALENDAR TABLE */
.yui-calendar {
	position:relative;
}

/* NAVBAR LEFT ARROW CONTAINER */
.yui-calendar .calnavleft {
	position:absolute;
	z-index:1;
}

/* NAVBAR RIGHT ARROW CONTAINER */
.yui-calendar .calnavright {
	position:absolute;
	z-index:1;
}

/* NAVBAR TEXT CONTAINER */
.yui-calendar .calheader {
	position:relative;
	width:100%;
	text-align:center;
}

/* Specific changes for calendar running under fonts/reset */
.yui-calendar .calbody a:hover {background:inherit;}
p#clear {clear:left; padding-top:10px;}