#!/usr/local/bin/perl

use strict;
use warnings;

use Test::More;

use constant TEST_COUNT     => 1;    # How many tests will be run in this script?

plan(tests => TEST_COUNT);

ok(1, "Solstice test scheme does not yet work with CPAN");


