<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xml:lang="<!-- sol_var document_lang -->" lang="<!-- sol_var document_lang -->">
  <head>
    <title><!-- sol_var document_title --></title>
    <base id="solstice_base" href="<!-- sol_var document_base -->"/>
    <meta http-equiv="Expires" content="0"/>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
    <meta http-equiv="Cache-Control" content="no-cache"/>
    <meta http-equiv="Pragma" content="no-cache"/>

    <script type="text/javascript">
        solstice_document_base      = '<!-- sol_var document_base -->';
        solstice_session_app_key    = '<!-- sol_var session_app_key -->';
        solstice_subsession         = '<!-- sol_var subsession_id -->';
        solstice_subsession_chain   = '<!-- sol_var chain_id -->';
        solstice_app_path           = '<!-- sol_var app_path -->';
    </script>
    <!-- sol_loop document_script -->
    <script type="text/javascript" src="<!-- sol_var src -->"></script><!-- /sol_loop -->
    
    <script type="text/javascript">
      function initApplication() {<!-- sol_loop onload_events -->
        <!-- sol_var event -->;<!-- /sol_loop -->
        Solstice.initializeToolTips();
      }<!-- sol_if escape_frames -->
      Solstice.escapeFrames();<!-- /sol_if -->
      Solstice.development_mode = <!-- sol_var development_mode -->;
    </script>
    <!-- sol_loop document_stylesheet -->
    <link rel="stylesheet" type="text/css" href="<!-- sol_var src -->" title="Solstice Style Sheet"/><!-- /sol_loop -->
  </head>

  <body onload="initApplication();">
    <form action="<!-- sol_var uri -->" method="post" accept-charset="UTF-8" id="solstice_app_form" onsubmit="return Solstice.Button.performClientAction();">
      <input type="hidden" id="solstice_selected_button" name="solstice_selected_button" value=""/>
      <a id="solstice_focus_anchor" href="<!-- sol_var uri -->" style="position: absolute; top: -100px; left: -100px;"></a>
      <script type="text/javascript">document.getElementById('solstice_focus_anchor').focus();</script>
      
      <!-- BEGIN BOILERPLATE WORKSPACE -->
      <!-- sol_var boilerplate -->
      <!-- END BOILERPLATE WORKSPACE -->

      <!-- sol_var developer_toolbar --> 
    </form>
  </body>
</html>
