<div class="soldevtoolbar"<!-- sol_if is_dev_mode --> style="display: block;"<!-- /sol_if -->><!-- sol_if is_dev_mode -->
    <div style="float: right;"><!-- sol_var wire_frames_button --> <!-- sol_var lang_tags_button --></div>
    <span>SQL: ___SOLSTICE_DB_READ_COUNT___ read, ___SOLSTICE_DB_WRITE_COUNT___ write</span><!-- /sol_if -->
    <span>Process: <!-- sol_var process --></span>
    <span>Time: ___SOLSTICE_PAGE_LOAD_TIME___ sec</span>
</div>
