/*
Copyright (c) 2007, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.3.0
*/
/* logger default styles */
/* default width: 31em */
/* default font-size 77% */
.yui-log {padding:1em;width:31em;background-color:#AAA;color:#000;border:1px solid black;font-family:monospace;font-size:77%;text-align:left;z-index:9000;}

/* for containers built from scratch */
.yui-log-container {position:absolute;top:1em;right:1em;}

/* buttons */
.yui-log input {
    margin:0;padding:0;
    font-family:arial;
    font-size:100%;
    font-weight:normal;
}
.yui-log .yui-log-btns {position:relative;float:right;bottom:.25em;}

/* header */
.yui-log .yui-log-hd {margin-top:1em;padding:.5em;background-color:#575757;}
.yui-log .yui-log-hd h4 {margin:0;padding:0;font-size:107%;font-weight:bold;color:#FFF;}

/* body */

.yui-log .yui-log-bd {width:100%;height:20em;background-color:#FFF;border:1px solid gray;overflow:auto;} /* height is controlled here: default 20em*/
.yui-log p {margin:1px;padding:.1em;}
.yui-log pre {margin:0;padding:0;}

/* for pre to respect newlines yet wrap long lines */
/* http://www.longren.org/2006/09/27/wrapping-text-inside-pre-tags/ */
.yui-log pre.yui-log-verbose {
    white-space: pre-wrap; /* css-3 */
    white-space: -moz-pre-wrap !important; /* Mozilla, since 1999 */
    white-space: -pre-wrap; /* Opera 4-6 */
    white-space: -o-pre-wrap; /* Opera 7 */
    word-wrap: break-word; /* Internet Explorer 5.5+ */
}

/* footer */
.yui-log .yui-log-ft {margin-top:.5em;}
.yui-log .yui-log-ft .yui-log-categoryfilters {}
.yui-log .yui-log-ft .yui-log-sourcefilters {width:100%;border-top:1px solid #575757;margin-top:.75em;padding-top:.75em;}
.yui-log .yui-log-filtergrp {margin-right:.5em;}
/*.yui-log .yui-log-ft ul {margin:0;padding:0;line-height:1.8}
.yui-log .yui-log-ft li {list-style:none;display:inline;white-space:nowrap;}*/

/* logs */
.yui-log .info {background-color:#A7CC25;} /* A7CC25 green */
.yui-log .warn {background-color:#F58516;} /* F58516 orange */
.yui-log .error {background-color:#E32F0B;} /* E32F0B red */
.yui-log .time {background-color:#A6C9D7;} /* A6C9D7 blue */
.yui-log .window {background-color:#F2E886;} /* F2E886 tan */
