/*
Copyright (c) 2007, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.3.0
*/
.yui-button  {

    display:-moz-inline-box; /* Gecko */
    display:inline-block; /* IE, Opera and Safari */
    border-width:1px 0;
    border-style:solid;
    border-color:#999;
    background:#ecece3 url(background.png) left center;
    margin:auto .25em;
    
}

.yui-button.ie6 {

    /* Give the transparent background image to IE 6 */
    background-image:none;
    filter:progid:DXImageTransform.Microsoft.AlphaImageLoader(src='../../build/button/assets/background.png', sizingMethod = 'scale');

}

.yui-button .first-child {

    display:block;
    *display:inline-block; /* IE */
    border-width:0 1px;
    border-style:solid;
    border-color:#999;
    margin:0 -1px;
    *position:relative;
    *left:-1px;

}

.yui-button button,
.yui-button a {

    display:block;
    *display:inline-block; /* IE */
    padding:.25em .5em;
    border:1px solid #ccc;

}

.yui-button button {

    *overflow:visible; /* Remove superfluous padding for IE */
    font-size:100%; /* Makes form controls resizable in IE */
    background-color:transparent;
    cursor:pointer;
    cursor:hand;

}

.yui-button a {

    text-decoration:none;
    color:#000;           

}

.yui-split-button button,
.yui-menu-button button {

    padding-right:20px;
    background-position:right center;
    background-repeat:no-repeat;
    
}

.yui-menu-button button {

    background-image:url(menuarrow.gif);

}

.yui-split-button button {

    background-image:url(splitarrow.gif);

}


/* Focus state */

.yui-button-focus {

    border-color:#5e5c95;

}

.yui-button-focus .first-child {

    border-color:#5e5c95;

}

.yui-button-focus button, 
.yui-button-focus a {

    border-color:#cec1fc;

}


/* Hover state */

.yui-button-hover {

    border-color:#406fac;
    background-color:#98d5fc;

}

.yui-button-hover .first-child {

    border-color:#406fac;

}

.yui-button-hover button, 
.yui-button-hover a {

    border-color:#7099ce;

}


/* Active state */

.yui-button-active {
    
    border-color:#7a8180;
    background-color:#333;
    
}

.yui-button-active .first-child {

    border-color:#7a8180;

}

.yui-button-active button, 
.yui-button-active a {

    border-color:#98a09f;

}

.yui-split-button-activeoption button {

    background-color:transparent;
    background-image:url(splitarrow_active.gif);
    
}



/* Checked state */

.yui-radio-button-checked,
.yui-checkbox-button-checked {
    
    border-color:#7a8180;
    background-color:#333;
    
}

.yui-radio-button-checked .first-child,
.yui-checkbox-button-checked .first-child {

    border-color:#7a8180;

}

.yui-radio-button-checked button,
.yui-checkbox-button-checked button { 

    border-color:#98a09f;

}



/* Disabled state */

.yui-button-disabled {
    
    border-color:#cbcdc5;
    background:#ecece3;

    filter:none;
    
}

.yui-button-disabled .first-child {

    border-color:#cbcdc5;

}

.yui-button-disabled button, 
.yui-button-disabled a {

    border-color:transparent;
    color:#b9b9b9;
    cursor:default;

}