<div class="sol_breadcrumbs">
    <!-- sol_if has_title_button -->
    <h1><!-- sol_var title_button --><!-- sol_if description --> <!-- sol_var description --> <!-- /sol_if --></h1>
    <!-- /sol_if -->
    <div>
        <!-- sol_loop buttons -->
        <!-- sol_var button -->
        <!-- sol_unless __LAST__ -->
        <span class="sol_gt">&gt;</span>
        <!-- /sol_unless --><!-- /sol_loop -->
    </div>
</div>
