/*
Copyright (c) 2007, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.3.0
*/
/* MenuBar style rules */

.yui-skin-sam .yuimenubar {

    border: solid 1px #808080;
    background: url(../../../../assets/skins/sam/sprite.png) repeat-x 0 0;

}

/* 
    The following two style rules cause a MenuBar's UL to clear its 
    floating LIs.
*/

.yui-skin-sam .yuimenubar .bd,
.yui-skin-sam .yuimenubar ul {

    *zoom: 1;   /* For IE */

}

.yui-skin-sam .yuimenubar>.bd>ul:after {

    content: ".";
    display: block;
    clear: both;
    visibility: hidden;
    height: 0;
    line-height: 0;

}


/* MenuBarItem style rules */

.yui-skin-sam .yuimenubaritem {

    float: left;

}

.yui-skin-sam .yuimenubarnav .yuimenubaritem {

    border-right: solid 1px #ccc;

}

.yui-skin-sam .yuimenubaritemlabel {

    display: block;
    *display: inline-block; /* For IE */
    font-size: 93%;  /* 12px */
    line-height: 2;  /* ~24px */
    *line-height: 1.9; /* For IE */
    padding: 0 10px;
    color: #000;
    text-decoration: none;
    outline: none;
    cursor: default;
    border-style: solid;
    border-color: #808080;
    border-width: 1px 0;
    position: relative;
    margin: -1px 0;

}

.yui-skin-sam .yuimenubarnav .yuimenubaritemlabel {

    padding-right: 20px;

}

.yui-skin-sam .yuimenubaritemlabel .submenuindicator {
    
    /*
        Set height and width to 1px so screen readers announce the content of 
        the element.
    */

    width: 1px;
    height: 1px;
    top: 0;
    left: 0;

}

.yui-skin-sam .yuimenubarnav .yuimenubaritemlabel .submenuindicator {
    
    top: 50%;
    right: 8px;
    left: auto; /* Override "left" property set by previous style rule */
    margin-top: -3px;
    height: 4px;
    width: 7px;
    text-indent: 8px;
    background-position: -16px -856px;

}


/* MenuBarItem states */

/* Selected MenuBarItem */

.yui-skin-sam .yuimenubaritem a.selected {

    background: url(../../../../assets/skins/sam/sprite.png) repeat-x 0 -1700px;
    border-color: #7D98B8;

}

.yui-skin-sam .yuimenubarnav .yuimenubaritem a.selected {

    border-left-width: 1px;
    margin-left: -1px;
    *left: -1px;    /* For IE */

}

.yui-skin-sam .yuimenubar a.selected .submenuindicator {

    background: transparent;

}

.yui-skin-sam .yuimenubarnav a.selected .submenuindicator {

    /*
        Reset the value of the "background" property since it is set to 
        "transparent" by the previous rule.
    */

    background: url(../../../../assets/skins/sam/sprite.png) repeat-x -16px -856px;

}


/* Disabled  MenuBarItem */

.yui-skin-sam .yuimenubaritem a.disabled {

    cursor: default;
    color: #A6A6A6;

}

.yui-skin-sam .yuimenubarnav .yuimenubaritem a.disabled .submenuindicator {

    background-position: -16px -881px;

}


/* Menu style rules */

.yui-skin-sam .yuimenu .bd {

    position: relative;
    top: 0;
    left: 0;
    border: solid 1px #808080;
    background-color: #fff;
    z-index: 1;
    
}

.yui-skin-sam .yuimenu ul {

    padding: 3px 0;
    border-width: 1px 0 0 0;
    border-color: #ccc;
    border-style: solid;

}

.yui-skin-sam .yuimenu ul.first-of-type {

    border-width: 0;

}


/* Group titles */

.yui-skin-sam .yuimenu h6 { 

    font-size: 93%;  /* 12px */
    font-weight: bold;
    line-height: 1.5;  /* 18px */
    *line-height: 1.45; /* For IE */
    border-style: solid;
    border-color: #ccc;
    border-width: 1px 0 0 0;
    color: #a4a4a4;    
    padding: 3px 10px 0 10px;

}

.yui-skin-sam .yuimenu ul.hastitle,
.yui-skin-sam .yuimenu h6.first-of-type {

    border-width: 0;

}


/* Top and bottom scroll controls */

.yui-skin-sam .yuimenu .topscrollbar,
.yui-skin-sam .yuimenu .bottomscrollbar {

    position: relative;
    height: 16px;
    border: solid 1px #808080;
    background: #fff url(../../../../assets/skins/sam/sprite.png) no-repeat 0 0;

}

.yui-skin-sam .yuimenu .topscrollbar {

    z-index: 2;
    border-bottom-color: #ccc;
    margin-bottom: -1px;
    background-position: center -950px;

}

.yui-skin-sam .yuimenu .topscrollbar_disabled {

    background-position: center -975px;

}

.yui-skin-sam .yuimenu .bottomscrollbar {

    z-index: 3;
    border-top-color: #ccc;
    margin-top: -1px;
    background-position: center -850px;

}

.yui-skin-sam .yuimenu .bottomscrollbar_disabled {

    background-position: center -875px;

}


/* MenuItem style rules */

.yui-skin-sam .yuimenuitemlabel {

    font-size: 93%;  /* 12px */
    line-height: 1.5;  /* 18px */
    *line-height: 1.45; /* For IE */
    padding: 0 20px;
    display: block;
    color: #000;
    text-decoration: none;
    outline: none;
    position: relative;
    cursor: default;

}

.yui-skin-sam .yuimenuitemlabel .helptext {

    font-style: normal;
    margin: 0 0 0 40px;
    
}

.yui-skin-sam .yuimenuitemlabel .submenuindicator,
.yui-skin-sam .yuimenuitemlabel .checkedindicator,
.yui-skin-sam .yuimenubaritemlabel .submenuindicator {
    
    position: absolute;
    overflow: hidden;
    background: url(../../../../assets/skins/sam/sprite.png) no-repeat;

}

.yui-skin-sam .yuimenuitemlabel .submenuindicator {

    top: 50%;
    right: 8px;
    margin-top: -3px;
    height: 7px;
    width: 4px;
    text-indent: 5px;
    background-position: 0 -906px;

}

.yui-skin-sam .yuimenuitemlabel .checkedindicator {

    top: 50%;
    left: 8px;
    margin-top: -3px;
    height: 7px;
    width: 7px;
    text-indent: 8px;
    background-position: 0 -1006px;

}


/* Menu states */

/* Visible Menu */

.yui-skin-sam .yui-menu-shadow-visible {

    top: 2px;
    right: -3px;
    left: -3px;
    bottom: -3px;
    background-color: #000;
    opacity: .12;
    *filter: alpha(opacity=12);  /* For IE */
    visibility: visible;

}

.yui-skin-sam .visible .bd,
.yui-skin-sam .visible .yuimenuitem {

    _zoom: 1;   /* For IE 7 Quirks Mode and IE 6 Standards and Quirks Mode */

}

.yui-skin-sam .visible .yuimenuitemlabel {

    *zoom: 1;   /* For IE */

}

.yui-skin-sam .visible .yuimenuitemlabel .helptext {

    float: right;
    width: 100%;
    text-align: right;
    margin: -1.5em 0 0 0;
    *margin: -1.45em 0 0 0;  /* For IE*/

}


/* MenuItem states */

/* Selected MenuItem */

.yui-skin-sam .yuimenuitem a.selected {

    /*
        Use "background" over "background-color" to clear background image 
        set by the ".yuimenubaritem a.selected" rule.
    */

    background: #B3D4FF;

}

/*
    Redefine the background for MenuItem instances that are children of
    a Menubar since the 
    ".yui-skin-sam .yuimenubar a.selected .submenuindicator" rule sets the 
    "background" property to "transparent"
*/

.yui-skin-sam .yuimenubar .yuimenuitem a.selected .submenuindicator {

    background: url(../../../../assets/skins/sam/sprite.png) no-repeat 0 -906px;

}

/*
    Remove borders and margin added by the style rule 
    ".yui-skin-sam .yuimenubarnav .yuimenubaritem a.selected"
*/

.yui-skin-sam .yuimenubarnav .yuimenuitem a.selected {

    border-width: 0;
    margin: 0;
    *left: 0;   /* For IE */

}


/* Disabled MenuItem */

.yui-skin-sam .yuimenuitem a.disabled {

    cursor: default;
    color: #A6A6A6;

}

.yui-skin-sam .yuimenuitem a.disabled .submenuindicator {

    background-position: 0 -931px;

}

.yui-skin-sam .yuimenuitem a.disabled .checkedindicator {

    background-position: 0 -1031px;

}