<!-- sol_if name=messages -->
<div id="sol_message_service_container" class="sol_message_service_container sol_message_service_information">
    <table border="0" cellpadding="0" cellspacing="0">
        <tr>
            <td class="sol_message_service_icon" rowspan="3"></td>
            <td><div class="sol_message_service_container_level1"></div><div class="sol_message_service_container_level2"></div></td>
        </tr>
        <tr>
            <td class="sol_message_service_text">
                <!-- sol_loop name=messages -->
                <!-- sol_var name=msg --><br />
                <!-- /sol_loop -->
            </td>
        </tr>
        <tr><td><div class="sol_message_service_container_level2"></div><div class="sol_message_service_container_level1"></div></td></tr>
    </table>
</div>
<!-- /sol_if -->
