/*
Copyright (c) 2007, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 2.3.0
*/
.yui-log {padding-top:3em;}
/*
.yui-log-container {position:relative;width:60em}
.yui-log .yui-log-bd {height:auto; overflow:visible;}
*/
.yui-log-container {width:40em}
.yui-log .yui-log-bd {height:60em}
.yui-log .yui-log-btns {display:none;}
.yui-log .yui-log-ft .yui-log-sourcefilters {visibility:hidden;}
.yui-log .yui-log-hd {display:none;}
.yui-log .yui-log-ft {position:absolute;top:0em;}

.pass {
    background-color: green;
    font-weight: bold;
    color: white;
}
.fail {
    background-color: red;
    font-weight: bold;
    color: white;
}
.ignore {
    background-color: #666;
    font-weight: bold;
    color: white;
}
