<!-- <div id="solstice_popin">
    <div class="hd" id="solstice_popin_title">
    </div>
    <div id="solstice_popin_content" class="bd"><img src="images/processing.gif" alt="">
        <form name="frm_solstice_popin" action=""></form>
        this is requirement for yahooui... it was eating my form tags unless i had more than 1 -jr
        <form name="frm_solstice_popin3" action=""></form>
        </div>
    </div>-->
