<input type="text" id="<!-- sol_var name=name -->" name="<!-- sol_var name=name -->" value="<!-- sol_var name=content -->" style="width: <!-- sol_var name=width -->;"<!-- sol_if name=char_limit --> maxlength="<!-- sol_var name=char_limit -->"<!-- /sol_if -->/> <!-- sol_var name=error -->
